# Cohoba - a GNOME client for Telepathy
#
# Copyright (C) 2006 Collabora Limited
#
# This package is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This package is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this package; if not, write to the Free Software
# Foundation, 51 Franklin Street, Fifth Floor, Boston, MA, 02110-1301 USA.

from os.path import join
import gtk
from gettext import gettext as _

import cohoba
from cohoba.common.Utils import load_icon, global_error_handler
from cohoba.contacts.MergedContactList import get_contact_list, MergedContactList
from cohoba.contacts.Contact import Contact
from cohoba.contacts.ContactGroup import ContactGroup, OverviewContactGroup, get_group_filter
from cohoba.contacts.ui.ContactListWindowManager import get_contact_list_window_manager
from cohoba.contacts.ui.ContactListView import ContactListView		
from cohoba.common.ui.PresenceComboBox import PresenceComboBox
from cohoba.common.DBusProxyProvider import get_proxy_provider
from cohoba.common.TelepathyConstants import (MISSION_CONTROL_UI_INTERFACE,
	MISSION_CONTROL_OBJECT_PATH, MISSION_CONTROL_BUS_NAME)

class CohobaAppletMainWindow:
	def __init__(self):
		glade = gtk.glade.XML(join(cohoba.SHARED_DATA_DIR, "main.glade"), "main")
		self.main = glade.get_widget("main")
		self.main.hide()
		
		self.contacts = ContactListView(OverviewContactGroup())
		self.contacts.show()
		glade.get_widget("contacts_scroll").add(self.contacts)
		
		self.message = glade.get_widget("message")
	
		self.presence_combobox = PresenceComboBox(self.message)
		self.presence_combobox.show()
		glade.get_widget("presence_table").pack_start(self.presence_combobox, False)
		
		self.main.connect('delete-event', self.on_window_delete)
	
	def on_window_delete(self, win, event):
		self.main.hide()
		return True

class CohobaAppletWidget:
	def __init__(self, applet):
		self.applet = applet
		
		self.applet.connect("button-press-event", self.on_applet_button_press)
		self.applet.setup_menu_from_file (
			cohoba.SHARED_DATA_DIR, "Cohoba.xml",
			None, [
			("About", self.on_about),
			("Accounts", self.on_accounts),
			])
			
	def make_menu(self, event):
		raise NotImplementedError
		
	def on_about (self, component, verb):
		print 'FIXME: to be implemnted', component, verb
	
	def on_accounts (self, component, verb):
		mission_control_ui_iface = get_proxy_provider().get_iface(
				MISSION_CONTROL_OBJECT_PATH,
				MISSION_CONTROL_UI_INTERFACE)
		mission_control_ui_iface.ShowAccounts(reply_handler=lambda: None, error_handler=global_error_handler)
								
	def on_applet_button_press(self, widget, event):
		if event.button == 1:
			self.make_menu(event)
#			self.main_ui.main.present_with_time(event.time)
			return True
			
		return False

class CohobaIconAppletWidget(CohobaAppletWidget, gtk.EventBox):
	def __init__(self, applet, icon=None):
		CohobaAppletWidget.__init__(self, applet)
		gtk.EventBox.__init__(self)
		self.image = gtk.Image()
		self.pixbuf = load_icon("cohoba.png")
		pixbuf = self.pixbuf.scale_simple(applet.get_size(), applet.get_size(), gtk.gdk.INTERP_BILINEAR)
		self.image.set_from_pixbuf(pixbuf)
		self.add(self.image)

		self.connect("size-allocate", self.on_size_allocate)
		applet.connect("change-orient", self.on_change_orient)
		applet.connect("change-size", self.on_change_size)
		applet.set_background_widget(self)
		
	def on_size_allocate(self, widget, allocation):
		if self.image.get_pixbuf() == None or (allocation.width != self.image.get_pixbuf().get_width() or allocation.height != self.image.get_pixbuf().get_height()):
			pixbuf = self.pixbuf.scale_simple(allocation.width, allocation.height, gtk.gdk.INTERP_BILINEAR)
			self.image.set_from_pixbuf(pixbuf)
	
	def on_change_orient(self, applet, orient):
		pass
		
	def on_change_size(self, applet, size):
		pixbuf = self.pixbuf.scale_simple(size, size, gtk.gdk.INTERP_BILINEAR)
		self.image.set_from_pixbuf(pixbuf)

	
class CohobaGroupIconAppletWidget(CohobaIconAppletWidget):
	def __init__(self, applet):
		CohobaIconAppletWidget.__init__(self, applet, "cohoba.png")
		self.group = OverviewContactGroup()
		self.main_ui = CohobaAppletMainWindow()
	
	def set_group(self, group):
		self.group = group
		self.image.set_from_icon_name("stock_people", gtk.ICON_SIZE_BUTTON)
	
	def make_menu(self, event):
		menu = gtk.Menu()
		menu.attach_to_widget(self, None)

		item = gtk.MenuItem(_("Choose Group"))
		menu.append(item)
		
		submenu = gtk.Menu()
		item.set_submenu(submenu)

		menu.append(gtk.SeparatorMenuItem())		

		def group_clicked(item, group):
			self.set_group(group)
			menu.popdown()
			
		for group in [row[0] for row in get_group_filter(OverviewContactGroup(), get_contact_list()) if row[0].is_group]:
			label = gtk.Label()
			label.set_markup(group.get_description())
			label.set_alignment(0.0, 0.0)
					
			image = gtk.Image()
			image.set_from_pixbuf(group.get_icon())

			box = gtk.HBox(False, 6)
			box.pack_start(image, expand=False)
			box.pack_start(label)

			item = gtk.MenuItem()
			item.add(box)		
			item.connect('activate', group_clicked, group)
			submenu.append(item)
		
		def contact_clicked(item, contact):
			contact.action()
			menu.popdown()
		
		# ------ Start horrible Hack to prevent apparition of a Assertion Failure when dealing with
		# python iteration of the model iter->stamp stuff.
		# As a workaround use the foreach() of the model
		flst = get_group_filter(self.group, get_contact_list())
		def append_items(model, path, iter, data):
			contact = model[path][0]
			if not contact.is_group and self.group.is_in_group(contact):
				label = gtk.Label()
				label.set_markup(contact.get_description())
				label.set_alignment(0.0, 0.0)
						
				image = gtk.Image()
				image.set_from_pixbuf(contact.get_icon())

				box = gtk.HBox(False, 6)
				box.pack_start(image, expand=False)
				box.pack_start(label)

				item = gtk.MenuItem()
				item.add(box)		
				item.connect('activate', contact_clicked, contact)
				menu.append(item)
		flst.foreach(append_items, None)
							
		menu.show_all()
		menu.popup(None, None, None, event.button, event.time)			
		
class CohobaMainIconAppletWidget(CohobaIconAppletWidget):
	def __init__(self, applet):
		CohobaIconAppletWidget.__init__(self, applet, "cohoba.png")
		self.main_ui = CohobaAppletMainWindow()
		
	def make_menu(self, event):
		menu = gtk.Menu()
		menu.attach_to_widget(self, None)
		
		def contact_clicked(item, contact):
			contact.action()
			menu.popdown()
		
		label = gtk.Label()
		label.set_markup(_("Open Main Window"))
		label.set_alignment(0.0, 0.0)
				
		image = gtk.Image()
		image.set_from_icon_name("stock_people", gtk.ICON_SIZE_DIALOG)

		box = gtk.HBox(False, 6)
		box.pack_start(image, expand=False)
		box.pack_start(label)

		item = gtk.MenuItem()
		item.add(box)
		item.connect('activate', lambda item: self.main_ui.main.present_with_time(gtk.get_current_event_time()))
		menu.append(item)
		
		menu.append(gtk.SeparatorMenuItem())
		
		for contact in [row[0] for row in get_group_filter(OverviewContactGroup(), get_contact_list())]:
			label = gtk.Label()
			label.set_markup(contact.get_description())
			label.set_alignment(0.0, 0.0)
					
			image = gtk.Image()
			image.set_from_pixbuf(contact.get_icon())

			box = gtk.HBox(False, 6)
			box.pack_start(image, expand=False)
			box.pack_start(label)

			item = gtk.MenuItem()
			item.add(box)		
			item.connect('activate', contact_clicked, contact)
			menu.append(item)	
		
		menu.show_all()
		menu.popup(None, None, None, event.button, event.time)
