#!/usr/bin/env python
#
# Cohoba - a GNOME client for Telepathy
#
# Copyright (C) 2006 Collabora Limited
#
# This package is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This package is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this package; if not, write to the Free Software
# Foundation, 51 Franklin Street, Fifth Floor, Boston, MA, 02110-1301 USA.


import getopt, sys
from os.path import exists, isdir, isfile, join, dirname, abspath

def usage():
	print """=== Cohoba: Usage
$ cohoba-applet [OPTIONS] component

OPTIONS:
	-h, --help			Print this help notice.
	-w, --window		Launch the applet in a standalone window for test purposes (default=no).
	-t, --trace			Use tracing
	-m, --mode			Applet mode, one of 'group', 'main', 'contact' (default='main')
	"""
	sys.exit()

# --------Start Bootstrap code ---------
# Allow to use uninstalled
def _check(path):
	return exists(path) and isdir(path) and isfile(path+"/AUTHORS")

name = join(dirname(__file__), '..')
if _check(name):
	print 'Running uninstalled cohoba, modifying PYTHONPATH'
	sys.path.insert(0, abspath(name))
else:
	sys.path.insert(0, abspath("@PYTHONDIR@"))
	print "Running installed cohoba, using [@PYTHONDIR@:$PYTHONPATH]"

# Now the path is set, import our stuff
import cohoba, cohoba.defs

import gettext, locale
gettext.bindtextdomain('cohoba', abspath(join(cohoba.defs.DATA_DIR, "locale")))
if hasattr(gettext, 'bind_textdomain_codeset'):
	gettext.bind_textdomain_codeset('cohoba','UTF-8')
gettext.textdomain('cohoba')

# Macosx doesn't seem to have locale.*
if hasattr(locale, 'bindtextdomain'):
	locale.bindtextdomain('cohoba', abspath(join(cohoba.defs.DATA_DIR, "locale")))
if hasattr(locale, 'bind_textdomain_codeset'):
	locale.bind_textdomain_codeset('cohoba','UTF-8')
if hasattr(locale, 'textdomain'):
	locale.textdomain('cohoba')

try:
	# attempt to set a name for killall
	import cohoba.osutils
	cohoba.osutils.set_process_name ("cohoba")
except Exception, e:
	print "Unable to set processName: %s" % e
# -------- End bootstrap code-----------

# Nice dialog to show uncaught exceptions
import cohoba.gtkexcepthook

def start_applet():
	import gnomeapplet
	
	from cohoba.CohobaFactory import CohobaFactory
	cohoba_factory = CohobaFactory()
	
	gnomeapplet.bonobo_factory(
		"OAFIID:Cohoba_Factory",
		gnomeapplet.Applet.__gtype__,
		cohoba.defs.PACKAGE,
		cohoba.defs.VERSION,
		cohoba_factory.make_applet)

# Return a standalone window that holds the applet
def standalone_window(mode):
	import gnome, gtk, gnomeapplet

	gnome.init(cohoba.defs.PACKAGE, cohoba.defs.VERSION)
		
	app = gtk.Window(gtk.WINDOW_TOPLEVEL)
	app.set_title("Cohoba")
	app.connect("destroy", gtk.main_quit)
	app.set_property('resizable', False)
	
	applet = gnomeapplet.Applet()
	applet.get_orient = lambda: gnomeapplet.ORIENT_DOWN
	
	from cohoba.CohobaFactory import CohobaFactory
	cohoba_factory = CohobaFactory()
	cohoba_factory.make_applet(applet, None, mode)

	applet.reparent(app)		
	app.show_all()
	
	return app

def houston():
	from cohoba.houston.Houston import Houston
	h = Houston()
	return h.exit
	
def technobabble():
	from cohoba.technobabble.TechnoBabbleChannelHandler import TechnoBabbleChannelHandler
	TechnoBabbleChannelHandler()
	
	from cohoba.bigbrother.BigBrotherChannelHandler import BigBrotherChannelHandler
	BigBrotherChannelHandler()

def run(do_trace, interrupt_callback=None):
	import gtk
	
	# run the new command using the given trace
	if do_trace:
		import trace
		trace = trace.Trace(
			ignoredirs=[sys.prefix],
			ignoremods=['sys', 'os', 'getopt', 'libxml2'],
			trace=True,
			count=False)
		trace.run('__import__("gtk").main()')
	else:
		try:
			gtk.main()
		except KeyboardInterrupt:
			if interrupt_callback!= None:
				interrupt_callback()
			else:
				raise
		
if __name__ == "__main__":
	from cohoba import COHOBA_APPLET_MODE_MAIN, COHOBA_APPLET_MODE_GROUP, COHOBA_APPLET_MODE_CONTACT
	
	standalone = False
	do_trace = False
	mode = COHOBA_APPLET_MODE_MAIN
	component = "applet"
	
	try:
		opts, args = getopt.getopt(sys.argv[1:], "hwtm:", ["help", "window", "trace", "mode"])
	except getopt.GetoptError:
		# Unknown args were passed, we fallback to bahave as if
		# no options were passed
		opts = []
		args = sys.argv[1:]
	
#	args.insert(0, sys.argv[0])
#	sys.argv = args
	
	for o, a in opts:
		if o in ("-h", "--help"):
			usage()
		elif o in ("-w", "--window"):
			standalone = True
		elif o in ("-t", "--trace"):
			do_trace = True
		elif o in ("-m", "--mode"):
			if a == "main":
				mode = COHOBA_APPLET_MODE_MAIN
			elif a == "group":
				mode = COHOBA_APPLET_MODE_GROUP
			elif a == "contact":
				mode = COHOBA_APPLET_MODE_CONTACT
			else:
				print 'Mode must be one of: main, group, contact.', a, ' is not recognized'
				usage()
		
	if len(args) >= 1 and not args[0].startswith("-"):
		component = args[0]
	
	print 'Runnin with options:', {
		'standalone': standalone,
		'do_trace': do_trace,
		'mode': mode,
		'component': component,
	}
	
	if component == "applet":
		if standalone:
			print 'Runing applet in standalone window'
			standalone_window(mode)
			run(do_trace)
		else:
			print 'Starting cohoba factory'
			start_applet()
	elif component == "houston":
		print 'Running houston mision control'
		run(do_trace, houston())
	elif component == "technobabble":
		print 'Running technobabble'
		technobabble()
		run(do_trace)
#	elif component == "houston+applet":
#		print 'Running Houston and Applet in standalone window in same process'
#		standalone_window(mode)
#		houston()
#		run(do_trace)
	else:
		print 'Error: Unknown component, try one of:\n\tapplet, houston'
		usage()
