# Cohoba - a GNOME client for Telepathy
#
# Copyright (C) 2006 Collabora Limited
#
# This package is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This package is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this package; if not, write to the Free Software
# Foundation, 51 Franklin Street, Fifth Floor, Boston, MA, 02110-1301 USA.

from telepathy.constants import CONNECTION_STATUS_DISCONNECTED
from telepathy.interfaces import CONN_INTERFACE, CHANNEL_HANDLER_INTERFACE

from cohoba.houston.AccountList import AccountList
from cohoba.common.Utils import global_error_handler, SignalConnection
from cohoba.common.DBusProxyProvider import get_proxy_provider
from cohoba.common.ChandlersRegistry import ChandlersRegistry

class ChannelHandler:
	def __init__(self, accounts):
		self.accounts = accounts
		self.chandlers = ChandlersRegistry()
		self.chandlers.LoadChandlers()
		self.handled_connections = []
		
	def handle_connection(self, conn_service, conn_obj):
		if conn_service in self.handled_connections:
			return
		
		self.handled_connections.append(conn_service)
		get_proxy_provider().create_proxy(conn_service, conn_obj)
		conn_iface = get_proxy_provider().get_iface(conn_obj, CONN_INTERFACE)
		
		print 'Listening connection:', conn_obj
		self.__StatusChanged_conn = SignalConnection(conn_iface, 'StatusChanged', lambda status, reason: self.on_status_changed(status, reason, conn_service))
		self.__NewChannel_conn = SignalConnection(conn_iface, 'NewChannel', lambda *channel: self.handle_channel(conn_service, conn_obj, *channel))
		conn_iface.ListChannels(reply_handler=lambda channels: self.on_channels_listed(conn_service, conn_obj, channels), error_handler=global_error_handler)
	
	def on_status_changed(self, status, reason, conn_service):
		if status == CONNECTION_STATUS_DISCONNECTED:
			self.handled_connections.remove(conn_service)
	
	def on_channels_listed(self, conn_service, conn_obj, channels):
		for channel in channels:
			chan_obj, chan_type, handle_type, handle = channel
			self.handle_channel(conn_service, conn_obj, chan_obj, chan_type, handle_type, handle)
	
	def handle_channel(self, conn_service, conn_obj, chan_obj, chan_type, handle_type, handle, suppress_handler=False):
		print 'New Channel: obj %r (type %r), handle (type %r) %r, suppress %r' % (chan_obj, chan_type, handle_type, handle, suppress_handler)

		if suppress_handler:
			print '** Handler suppressed:', chan_obj, chan_type, handle_type, handle, suppress_handler
			return

		handled = False
		
		chandlers = self.chandlers.GetChandlers(chan_type)
		print 'Available channel handler for type:', chan_type, chandlers
		for chandler in chandlers:
			get_proxy_provider().create_proxy(
				self.chandlers.GetBusName(chandler),
				self.chandlers.GetObjectPath(chandler))

			chandler_iface = get_proxy_provider().get_iface(
				self.chandlers.GetObjectPath(chandler),
				CHANNEL_HANDLER_INTERFACE)
			
			print 'Handling Channel with:', self.chandlers.GetBusName(chandler), self.chandlers.GetObjectPath(chandler), CHANNEL_HANDLER_INTERFACE
			chandler_iface.HandleChannel(conn_service, conn_obj, chan_type, chan_obj, handle_type, handle)
			handled = True

		if not handled:
			print '** New Channel Not handled', chan_obj, chan_type, handle_type, handle, suppress_handler
