import gtk, gtk.glade, gobject
from os.path import join
import cohoba

from cohoba.common.ui.ProfileComboBox import ProfileComboBox
from cohoba.common.Utils import singleton
from cohoba.houston.AccountList import AccountList
import cohoba.houston.AccountStore as AccountStore

class ProtocolParamsList(gtk.ListStore):
	def __init__(self, params):
		gtk.ListStore.__init__(self, gobject.TYPE_PYOBJECT, gobject.TYPE_STRING, gobject.TYPE_STRING)
		props = [(str, "server",),
		(str, "resource",),
		(str, "priority",),
		(int, "port",),
		(bool, "old-ssl",),
		(bool, "register",),
		(bool, "low-bandwitdth",),
		(str, "https-proxy-server",),
		(str, "https-proxy-port",),
		(str, "fallback-conference-server",),
		(str, "stun-server",),
		(int, "stun-port"),
		(bool, "ignore-ssl-errors",),
		]
		for t, key in props:
			value = ""
			print 'lets see if ', key, 'is in ', params
			if key in params:
				value = params[key]
			self.append([t, key, str(value)])
	
	def get_params(self):
		params = {}
		for t, key, val in self:
			try:
				if val != "":
					params[key] = t(val)
			except:
				pass
		
		return params
		
class AccountEditor(gtk.Window):
	def __init__(self, account):
		gtk.Window.__init__(self)
		
		self.account = account
		
		self.set_property("border_width", 12)
		self.set_property("window_position", gtk.WIN_POS_CENTER)
		
		glade = gtk.glade.XML(join(cohoba.SHARED_DATA_DIR, "accounts.glade"), "account_modification_contents")
		self.add(glade.get_widget("account_modification_contents"))
		
		self.account_entry = glade.get_widget("account")
		self.password = glade.get_widget("password")
		self.save_password = glade.get_widget("save_password")
		self.close = glade.get_widget("close")
		
		self.advanced_settings = glade.get_widget("advanced_settings")
		self.advanced_settings.set_model(ProtocolParamsList(account.params))
		cell = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Key", cell)
		column.set_attributes(cell, text=1)
		self.advanced_settings.append_column(column)
	
		cell = gtk.CellRendererText()
		cell.set_property('editable', True)
		cell.connect('edited', self.advanced_cell_edited)
		column = gtk.TreeViewColumn("Value", cell)
		column.set_attributes(cell, text=2)
		self.advanced_settings.append_column(column)
		
		self.profile = ProfileComboBox()
		glade.get_widget("table").attach(self.profile, 1, 2, 0, 1, xoptions=gtk.FILL, yoptions=gtk.FILL)
		self.profile.show()
		
		self.connect('destroy', self.on_close)
		self.close.connect("clicked", lambda widget: self.destroy())
		AccountList().connect('account-deleted', self.on_account_deleted)
		self.sync_account()
	
	def advanced_cell_edited(self, cell, path, new_text):
		model = self.advanced_settings.get_model()
		iter = model.get_iter(path)
		try:
			if new_text != "":
				model[iter][0](new_text)
			model.set(iter, 2, new_text)
		except Exception, e:
			print e
		
	def sync_account(self):
		self.profile.set_active_profile(self.account.profile)
		if self.account.get_username() != None:
			self.account_entry.set_text(self.account.get_username())
		if self.account.get_password() != None:
			self.password.set_text(self.account.get_password())
		self.save_password.set_active(self.account.save_password)
		
	def on_account_deleted(self, account_list, deleted_account):
		if deleted_account == self.account:
			self.destroy()
			
	def on_close(self, *args):
		active_profile = self.profile.get_active_profile()
		if active_profile != None:
			self.account.set_profile(active_profile)
		
		# Warning in the meantime we use this precise order, but ideally set_username should disappear
		self.account.set_params(self.advanced_settings.get_model().get_params())
		self.account.set_username(self.account_entry.get_text())
		self.account.set_password(self.password.get_text())
		
		self.account.set_save_password(self.save_password.get_active())
		
		if self.account not in AccountList():
			self.account.generate_name()
			AccountList().add_account(self.account)
		
class AccountEditorManager:
	def __init__(self):
		self.account_editors = {}

	def on_account_deleted(self, account):
		if account in self.account_editors:
			del self.account_editors[account]
							
	def get_editor_for_account(self, account=None):
		if account == None:
			account = AccountStore.create_new_account()
			
		if account not in self.account_editors:
			self.account_editors[account] = AccountEditor(account)
			self.account_editors[account].connect('destroy', lambda win: self.on_account_deleted(account))
		
		return self.account_editors[account]
	
	def show_account_editor(self, account=None):
		editor = self.get_editor_for_account(account)
		editor.present_with_time(gtk.get_current_event_time())
	
AccountEditorManager = singleton(AccountEditorManager)
